"""J.A.R.V.I.S. twitter background task handler."""
import logging
from datetime import datetime, timedelta

import tweepy
from dis_snek.ext.tasks.task import Task
from dis_snek.ext.tasks.triggers import IntervalTrigger
from jarvis_core.db import q
from jarvis_core.db.models import Twitter

import jarvis

logger = logging.getLogger("jarvis")


@Task.create(trigger=IntervalTrigger(minutes=1))
async def tweets() -> None:
    """J.A.R.V.I.S. twitter background task."""
    config = jarvis.config.get_config()
    __auth = tweepy.AppAuthHandler(
        config.twitter["consumer_key"], jarvis.jconfig.twitter["consumer_secret"]
    )
    __api = tweepy.API(__auth)
    guild_cache = {}
    channel_cache = {}
    twitters = Twitter.find(q(active=True))
    async for twitter in twitters:
        try:
            if not twitter.twitter_id or not twitter.last_sync:
                user = __api.get_user(screen_name=twitter.handle)
                twitter.update(
                    q(twitter_id=user.id, handle=user.screen_name, last_sync=datetime.now())
                )

            if twitter.last_sync + timedelta(hours=1) <= datetime.now():
                user = __api.get_user(id=twitter.twitter_id)
                twitter.update(q(handle=user.screen_name, last_sync=datetime.now()))

            if tweets := __api.user_timeline(id=twitter.twitter_id):
                guild_id = twitter.guild
                channel_id = twitter.channel
                tweets = sorted(tweets, key=lambda x: x.id)
                if guild_id not in guild_cache:
                    guild_cache[guild_id] = await jarvis.jarvis.get_guild(guild_id)
                guild = guild_cache[twitter.guild]
                if channel_id not in channel_cache:
                    channel_cache[channel_id] = await guild.fetch_channel(channel_id)
                channel = channel_cache[channel_id]
                for tweet in tweets:
                    retweet = "retweeted_status" in tweet.__dict__
                    if retweet and not twitter.retweets:
                        continue
                    timestamp = int(tweet.created_at.timestamp())
                    url = f"https://twitter.com/{twitter.handle}/status/{tweet.id}"
                    verb = "re" if retweet else ""
                    await channel.send(
                        f"`@{twitter.handle}` {verb}tweeted this at <t:{timestamp}:f>: {url}"
                    )
                newest = max(tweets, key=lambda x: x.id)
                twitter.update(q(last_tweet=newest.id))
                await twitter.commit()
        except Exception as e:
            logger.error(f"Error with tweets: {e}")
